require "store/memory/version"
require "store"

class Store
  class DataMapper
    class Memory < Store::DataMapper

      def initialize
        @items = {}
        @id = 0
      end

      def insert(data)
        id = next_id.to_s
        store(id, data)
        id
      end

      def update(id, data)
        id = id.to_s
        store(id, data)
        id
      end

      def delete(id)
        items.delete(id)
        id
      end

      def single_find(id)
        dup(items[id])
      end

      def bulk_find(ids)
        collection(items.values_at(*ids))
      end

      def all
        collection(items.values)
      end

      def count
        all.size
      end

      private

      def collection(items)
        @result = {}
        items.map do |data|
          @result.merge!(data['uid'] => dup(data))
        end
        @result
      end

      def items
        @items
      end

      def next_id
        @id += 1
      end

      def dup(data)
        data and HashDupper.dup(data)
      end

      class HashDupper
        def self.dup(hash)
          new(hash).dup
        end

        def initialize(hash)
          @hash = hash
        end

        def dup
          Hash.new.tap do |dupped_hash|
            @hash.each do |key, value|
              dupped_hash[key] = dup_obj(value)
            end
          end
        end

        private
        def dup_obj(obj)
          case obj
          when NilClass,Symbol,Numeric,TrueClass,FalseClass
            obj
          else
            obj.dup
          end
        end
      end

      def store(id, data)
        stored = dup(data)
        stored['uid'] = id
        @items[id] = stored
      end

    end
  end
end
